/* pi: Path from/to Id number conversion utility (non-cache version)
   in TBCppA project: tracer-based CPP analyzer.
   Copyright (c) 2006   Katsuhiko Gondow (gondow(at)cs.titech.ac.jp)
   $Id: pi.c,v 1.2 2006/11/01 11:36:57 gondow Exp gondow $ */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/param.h>
#include <string.h>
#include <assert.h>

#include "pi.h"
#include "misc.h"

#define PI_BUF_SIZE   ((path_max + 1) * 2)
/* ==================================================================== */
int   path_max;
/* ==================================================================== */
static FILE *pi_fp;
static char *pi_buf;
/* ==================================================================== */
void
pi_init (const char *db_file)
{
    pi_fp = fopen_with_mkdir (db_file, "a+"); /* do LF<->CRLF conversion */

    if (pi_fp == NULL) {
	perror ("fopen"); exit (1);
    }
#ifdef PATH_MAX
    path_max = PATH_MAX;
#else
    path_max = getconf (".", _PC_PATH_MAX);
#endif
    if (path_max < 1024)
	path_max = 1024;

    pi_buf = emalloc (PI_BUF_SIZE);
    assert (pi_buf != NULL);
}

void
pi_final (void)
{
    fclose (pi_fp);
    free (pi_buf);
    pi_buf = NULL;
}

void
pi_add  (const char *file, const char *path)
{
    rewind (pi_fp);
    while (fgets (pi_buf, PI_BUF_SIZE, pi_fp) != NULL) {
	chop_nl (pi_buf);
	if (!strncmp (path, pi_buf, strlen (path)))  /* same entry found! */ 
	    return;
    }
    fprintf (pi_fp, "%s,%s\n", path, file);
    fflush (pi_fp);
}

char *
pi_idnum2path (int idnum)
{
    int i;
    char *ret;

    rewind (pi_fp);
    for (i = 0; ; i++) {
	ret = fgets (pi_buf, PI_BUF_SIZE, pi_fp);
	if (ret == NULL)
	    break;
	if (i == idnum) {
	    ret = strchr (pi_buf, ',');
	    *ret = '\0';
	    return pi_buf; /* not reentrant */
	}
    }
    return NULL; /* no such path for idnum */
}

int
pi_path2idnum (const char *path)
{
    int i;
    char *ret;

    rewind (pi_fp);
    for (i = 0; ; i++) {
	ret = fgets (pi_buf, PI_BUF_SIZE, pi_fp);
	if (ret == NULL)
	    break;
	chop (pi_buf);
	if (!strncmp (path, pi_buf, strlen (path)))
	    return i; /* found */
    }

    return -1; /* not found */
}
/* ==================================================================== */
#if 0
int
main (int argc, char *argv [])
{
    pi_init ("test.db");
    pi_add ("foo", "/tmp/foo");
    pi_add ("/usr/include/stdio.h", "/usr/include/stdio.h");
    printf ("pi_path2idnum: %d\n", pi_path2idnum("/usr/include/stdio.h"));
    printf ("pi_path2idnum: %d\n", pi_path2idnum("/tmp/foo"));
    printf ("pi_path2idnum: %d\n", pi_path2idnum("/tmp/foo2"));
    printf ("pi_idnum2path: %s\n", pi_idnum2path(0));
    printf ("pi_idnum2path: %s\n", pi_idnum2path(1));
    printf ("pi_idnum2path: %s\n", pi_idnum2path(2));
    pi_final ();
}
#endif
/* ==================================================================== */
